@@ -1,8 +1,9 @@
-import icalendar
+from icalendar.tests import unittest
+
 import datetime
+import icalendar
 import os
 
-from . import unittest
 
 class TestTime(unittest.TestCase):
 
@@ -24,5 +25,5 @@ class TestTime(unittest.TestCase):
     def test_create_to_ical(self):
         cal = icalendar.Calendar()
         cal.add('X-SOMETIME', datetime.time(17, 20, 10))
-        self.assertTrue('X-SOMETIME;VALUE=TIME:172010' in
+        self.assertTrue(b'X-SOMETIME;VALUE=TIME:172010' in
                         cal.to_ical().splitlines())
